@@ -18,7 +18,7 @@ import numpy as np
 import pytest
 
 import pulser
-from pulser import Sequence, Pulse, Register
+from pulser import Sequence, Pulse, Register, Register3D
 from pulser.devices import Chadoq2, MockDevice
 from pulser.devices._device_datacls import Device
 from pulser.sequence import _TimeSlot
@@ -37,7 +37,7 @@ def test_init():
     with pytest.raises(TypeError, match="must be of type 'Device'"):
         Sequence(reg, Device)
 
-    fake_device = Device("fake", 2, 100, 100, 1, Chadoq2._channels)
+    fake_device = Device("fake", 2, 70, 100, 100, 1, Chadoq2._channels)
     with pytest.warns(UserWarning, match="imported from 'pulser.devices'"):
         Sequence(reg, fake_device)
 
@@ -70,7 +70,7 @@ def test_channel_declaration():
     seq2.declare_channel("ch0", "raman_local", initial_target="q1")
     seq2.declare_channel("ch1", "rydberg_global")
     seq2.declare_channel("ch2", "rydberg_global")
-    assert set(seq2.available_channels) == available_channels - {"mw_global"}
+    assert set(seq2.available_channels) == (available_channels - {"mw_global"})
     assert seq2._taken_channels == {
         "ch0": "raman_local",
         "ch1": "rydberg_global",
@@ -90,6 +90,51 @@ def test_channel_declaration():
         seq2.declare_channel("ch3", "rydberg_global")
 
 
+def test_magnetic_field():
+    seq = Sequence(reg, MockDevice)
+    with pytest.raises(
+        AttributeError,
+        match="only defined when the sequence " "is in 'XY Mode'.",
+    ):
+        seq.magnetic_field
+    seq.declare_channel("ch0", "mw_global")  # seq in XY mode
+    # mag field is the default
+    assert np.all(seq.magnetic_field == np.array((0.0, 0.0, 30.0)))
+    seq.set_magnetic_field(bx=1.0, by=-1.0, bz=0.5)
+    assert np.all(seq.magnetic_field == np.array((1.0, -1.0, 0.5)))
+    with pytest.raises(ValueError, match="magnitude greater than 0"):
+        seq.set_magnetic_field(bz=0.0)
+    assert seq._empty_sequence
+    seq.add(Pulse.ConstantPulse(100, 1, 1, 0), "ch0")
+    assert not seq._empty_sequence
+    with pytest.raises(ValueError, match="can only be set on an empty seq"):
+        seq.set_magnetic_field(1.0, 0.0, 0.0)
+
+    seq2 = Sequence(reg, MockDevice)
+    seq2.declare_channel("ch0", "rydberg_global")  # not in XY mode
+    with pytest.raises(ValueError, match="can only be set in 'XY Mode'."):
+        seq2.set_magnetic_field(1.0, 0.0, 0.0)
+
+    seq3 = Sequence(reg, MockDevice)
+    seq3.set_magnetic_field(1.0, 0.0, 0.0)  # sets seq to XY mode
+    assert set(seq3.available_channels) == {"mw_global"}
+    seq3.declare_channel("ch0", "mw_global")
+    # Does not change to default
+    assert np.all(seq3.magnetic_field == np.array((1.0, 0.0, 0.0)))
+    var = seq3.declare_variable("var")
+    # Sequence is marked as non-empty when parametrized too
+    seq3.add(Pulse.ConstantPulse(100, var, 1, 0), "ch0")
+    assert seq3.is_parametrized()
+    with pytest.raises(ValueError, match="can only be set on an empty seq"):
+        seq3.set_magnetic_field()
+
+    seq3_str = seq3.serialize()
+    seq3_ = Sequence.deserialize(seq3_str)
+    assert seq3_._in_xy
+    assert str(seq3) == str(seq3_)
+    assert np.all(seq3_.magnetic_field == np.array((1.0, 0.0, 0.0)))
+
+
 def test_target():
     seq = Sequence(reg, device)
     seq.declare_channel("ch0", "raman_local", initial_target="q1")
@@ -282,7 +327,9 @@ def test_sequence():
     seq.phase_shift(np.pi, "q0", basis="ground-rydberg")
 
     with patch("matplotlib.pyplot.show"):
-        seq.draw()
+        with patch("matplotlib.figure.Figure.savefig"):
+            seq.draw(fig_name="my_sequence.pdf")
+            seq.draw(draw_register=True, fig_name="both.pdf")
 
     pulse1 = Pulse(
         InterpolatedWaveform(500, [0, 1, 0]),
@@ -363,3 +410,147 @@ def test_sequence():
     assert json.loads(s)["__version__"] == pulser.__version__
     seq_ = Sequence.deserialize(s)
     assert str(seq) == str(seq_)
+
+
+def test_config_slm_mask():
+    reg_s = Register({"q0": (0, 0), "q1": (10, 10), "q2": (-10, -10)})
+    seq_s = Sequence(reg_s, device)
+
+    with pytest.raises(TypeError, match="must be castable to set"):
+        seq_s.config_slm_mask(0)
+    with pytest.raises(TypeError, match="must be castable to set"):
+        seq_s.config_slm_mask((0))
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_s.config_slm_mask("q0")
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_s.config_slm_mask(["q3"])
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_s.config_slm_mask(("q3",))
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_s.config_slm_mask({"q3"})
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_s.config_slm_mask([0])
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_s.config_slm_mask((0,))
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_s.config_slm_mask({0})
+
+    targets_s = ["q0", "q2"]
+    seq_s.config_slm_mask(targets_s)
+    assert seq_s._slm_mask_targets == {"q0", "q2"}
+
+    with pytest.raises(ValueError, match="configured only once"):
+        seq_s.config_slm_mask(targets_s)
+
+    reg_i = Register({0: (0, 0), 1: (10, 10), 2: (-10, -10)})
+    seq_i = Sequence(reg_i, device)
+
+    with pytest.raises(TypeError, match="must be castable to set"):
+        seq_i.config_slm_mask(0)
+    with pytest.raises(TypeError, match="must be castable to set"):
+        seq_i.config_slm_mask((0))
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_i.config_slm_mask("q0")
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_i.config_slm_mask([3])
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_i.config_slm_mask((3,))
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_i.config_slm_mask({3})
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_i.config_slm_mask(["0"])
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_i.config_slm_mask(("0",))
+    with pytest.raises(ValueError, match="exist in the register"):
+        seq_i.config_slm_mask({"0"})
+
+    targets_i = [0, 2]
+    seq_i.config_slm_mask(targets_i)
+    assert seq_i._slm_mask_targets == {0, 2}
+
+    with pytest.raises(ValueError, match="configured only once"):
+        seq_i.config_slm_mask(targets_i)
+
+
+def test_slm_mask():
+    reg = Register({"q0": (0, 0), "q1": (10, 10), "q2": (-10, -10)})
+    targets = ["q0", "q2"]
+    pulse1 = Pulse.ConstantPulse(100, 10, 0, 0)
+    pulse2 = Pulse.ConstantPulse(200, 10, 0, 0)
+
+    # Set mask when an XY pulse is already in the schedule
+    seq_xy1 = Sequence(reg, MockDevice)
+    seq_xy1.declare_channel("ch_xy", "mw_global")
+    seq_xy1.add(pulse1, "ch_xy")
+    seq_xy1.config_slm_mask(targets)
+    assert seq_xy1._slm_mask_time == [0, 100]
+
+    # Set mask and then add an XY pulse to the schedule
+    seq_xy2 = Sequence(reg, MockDevice)
+    seq_xy2.config_slm_mask(targets)
+    seq_xy2.declare_channel("ch_xy", "mw_global")
+    seq_xy2.add(pulse1, "ch_xy")
+    assert seq_xy2._slm_mask_time == [0, 100]
+
+    # Check that adding extra pulses does not change SLM mask time
+    seq_xy2.add(pulse2, "ch_xy")
+    assert seq_xy2._slm_mask_time == [0, 100]
+
+    # Check that SLM mask time is updated accordingly if a new pulse with
+    # earlier start is added
+    seq_xy3 = Sequence(reg, MockDevice)
+    seq_xy3.declare_channel("ch_xy1", "mw_global")
+    seq_xy3.config_slm_mask(targets)
+    seq_xy3.delay(duration=100, channel="ch_xy1")
+    seq_xy3.add(pulse1, "ch_xy1")
+    assert seq_xy3._slm_mask_time == [100, 200]
+    seq_xy3.declare_channel("ch_xy2", "mw_global")
+    seq_xy3.add(pulse1, "ch_xy2", "no-delay")
+    assert seq_xy3._slm_mask_time == [0, 100]
+
+    # Same as previous check, but mask is added afterwards
+    seq_xy4 = Sequence(reg, MockDevice)
+    seq_xy4.declare_channel("ch_xy1", "mw_global")
+    seq_xy4.delay(duration=100, channel="ch_xy1")
+    seq_xy4.add(pulse1, "ch_xy1")
+    seq_xy4.declare_channel("ch_xy2", "mw_global")
+    seq_xy4.add(pulse1, "ch_xy2", "no-delay")
+    seq_xy4.config_slm_mask(targets)
+    assert seq_xy4._slm_mask_time == [0, 100]
+
+    # Check that paramatrize works with SLM mask
+    seq_xy5 = Sequence(reg, MockDevice)
+    seq_xy5.declare_channel("ch", "mw_global")
+    var = seq_xy5.declare_variable("var")
+    seq_xy5.add(Pulse.ConstantPulse(200, var, 0, 0), "ch")
+    assert seq_xy5.is_parametrized()
+    seq_xy5.config_slm_mask(targets)
+    seq_xy5_str = seq_xy5.serialize()
+    seq_xy5_ = Sequence.deserialize(seq_xy5_str)
+    assert str(seq_xy5) == str(seq_xy5_)
+
+    # Check drawing method
+    with patch("matplotlib.pyplot.show"):
+        seq_xy2.draw()
+
+
+def test_draw_register():
+    # Draw 2d register from sequence
+    reg = Register({"q0": (0, 0), "q1": (10, 10), "q2": (-10, -10)})
+    targets = ["q0", "q2"]
+    pulse = Pulse.ConstantPulse(100, 10, 0, 0)
+    seq = Sequence(reg, MockDevice)
+    seq.declare_channel("ch_xy", "mw_global")
+    seq.add(pulse, "ch_xy")
+    seq.config_slm_mask(targets)
+    with patch("matplotlib.pyplot.show"):
+        seq.draw(draw_register=True)
+
+    # Draw 3d register from sequence
+    reg3d = Register3D.cubic(3, 8)
+    seq3d = Sequence(reg3d, MockDevice)
+    seq3d.declare_channel("ch_xy", "mw_global")
+    seq3d.add(pulse, "ch_xy")
+    seq3d.config_slm_mask([6, 15])
+    with patch("matplotlib.pyplot.show"):
+        seq3d.draw(draw_register=True)
